# Copyright 2025 Ant Group Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Custom exception types for the HTTP backend."""


class HttpBackendError(Exception):
    """Base exception for all HTTP backend errors."""


class ResourceNotFound(HttpBackendError):
    """Raised when a resource (session, computation, etc.) is not found."""


class InvalidRequestError(HttpBackendError, ValueError):
    """Raised for invalid requests, e.g., bad parameters or invalid state."""
